from django import forms
from django.contrib import admin
from django.contrib.admin.helpers import ACTION_CHECKBOX_NAME
from django.shortcuts import render, redirect
from django.utils.translation import gettext_lazy as _
from django.contrib import messages


def related_field_change_action(model, field_name, action_name):
    class FieldForm(forms.Form):
        _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)

        value = forms.ModelChoiceField(
            label=model._meta.verbose_name, queryset=model.objects.all()
        )

    def field_change_action(modeladmin, request, queryset):
        data = request.POST

        apply = "apply" in data

        initial = {"_selected_action": data.getlist(ACTION_CHECKBOX_NAME)}

        form = FieldForm(
            data=data if apply else None,
            initial=initial if not apply else None,
        )

        if apply and form.is_valid():
            queryset.update(**{field_name: form.cleaned_data["value"]})

            messages.success(request, _("Changes saved"))

            return redirect(request.get_full_path())

        context = admin.site.each_context(request)

        context.update(
            {
                "items": queryset,
                "form": form,
                "action_name": "field_change_action",
            }
        )

        return render(
            request, "admin/change_related_field_action.html", context
        )

    field_change_action.short_description = action_name

    return field_change_action



from django import template
from django.apps import apps
from django.http import HttpRequest

from cap.menu import MenuManager

register = template.Library()


@register.simple_tag(takes_context=True)
def get_cap_menu(context, request):
    if not isinstance(request, HttpRequest):
        return None
    available_apps = context.get("available_apps")
    if not available_apps:
        return None
    return MenuManager(available_apps, context, request)


@register.simple_tag()
def get_root_categories():
    return apps.get_model("categories", "Category").objects.root_nodes()



from django.contrib import admin
from django.shortcuts import render as django_render
from django.contrib.admin.views.decorators import staff_member_required


@staff_member_required
def render(request, template_name, context=None, status=None):
    ctx = admin.site.each_context(request)
    ctx.update(context or {})

    return django_render(request, template_name, ctx, status=status)



from django.utils.translation import gettext_lazy as _
from django.contrib.admin.filters import SimpleListFilter, FieldListFilter


class InputFilter(SimpleListFilter):
    template = "admin/input_filter.html"

    def lookups(self, request, model_admin):
        # Dummy, required to show the filter.
        return ((),)

    def choices(self, changelist):
        yield {
            'selected': self.value() is not None,
            'query_string': changelist.get_query_string({}, [self.parameter_name]),
            'parameter_name': self.parameter_name,
        }


class IsNullFieldListFilter(FieldListFilter):
    notnull_label = _("Is present")
    isnull_label = _("Is Null")

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.lookup_kwarg = "%s__isnull" % field_path
        self.lookup_val = request.GET.get(self.lookup_kwarg, None)

        super().__init__(
            field, request, params, model, model_admin, field_path
        )

    def expected_parameters(self):
        return [self.lookup_kwarg]

    def choices(self, cl):
        choices = (
            (None, _("All")),
            ("False", self.notnull_label),
            ("True", self.isnull_label),
        )
        for lookup, title in choices:
            yield {
                "selected": self.lookup_val == lookup,
                "query_string": cl.get_query_string(
                    {
                        self.lookup_kwarg: lookup,
                    }
                ),
                "display": title,
            }



from django.conf import settings
from django.forms import DateInput


class DatePickerInput(DateInput):
    def __init__(self, attrs=None, format=None):
        if format is None:
            format = settings.DATE_INPUT_FORMATS[0]

        if attrs is None:
            attrs = {}

        attrs["data-role"] = "datepicker"

        super().__init__(attrs=attrs, format=format)



from django.http.response import HttpResponseBase
from django.template.loader import render_to_string
from django.contrib.admin.views.decorators import staff_member_required

from cap.views import render


def admin_render_view(template_name):
    def decorator(view_func):
        @staff_member_required
        def wrapper(request, *args, **kwargs):
            context = view_func(request, *args, **kwargs)

            if isinstance(context, HttpResponseBase):
                return context

            return render(request, template_name, context)

        return wrapper

    return decorator


def short_description(description):
    def decorator(f):
        def wrap(*args, **kwargs):
            return f(*args, **kwargs)

        wrap.short_description = description
        wrap.__name__ = f.__name__

        return wrap

    return decorator


def template_list_item(template_name, description=None):
    def decorator(f):
        def wrap(*args, **kwargs):
            return render_to_string(template_name, f(*args, **kwargs))

        if description is not None:
            wrap.short_description = description

        return wrap

    return decorator
